/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity;

import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.ItemUtils;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2591;

public class TechStorageBaseBlockEntity extends MachineBaseBlockEntity
		implements InventoryProvider, IToolDrop, IListInfoProvider {

	public final int maxCapacity;
	public final RebornInventory<TechStorageBaseBlockEntity> inventory;
	public class_1799 storedItem;

	public TechStorageBaseBlockEntity(class_2591<?> blockEntityTypeIn, String name, int maxCapacity) {
		super(blockEntityTypeIn);
		this.maxCapacity = maxCapacity;
		storedItem = class_1799.field_8037;
		inventory = new RebornInventory<>(3, name, maxCapacity, this);
	}

	public void readWithoutCoords(class_2487 tagCompound) {

		storedItem = class_1799.field_8037;

		if (tagCompound.method_10545("storedStack")) {
			storedItem = class_1799.method_7915(tagCompound.method_10562("storedStack"));
		}

		if (!storedItem.method_7960()) {
			storedItem.method_7939(Math.min(tagCompound.method_10550("storedQuantity"), this.maxCapacity));
		}

		inventory.read(tagCompound);
	}

	public class_2487 writeWithoutCoords(class_2487 tagCompound) {
		if (!storedItem.method_7960()) {
			class_1799 temp = storedItem.method_7972();
			if (storedItem.method_7947() > storedItem.method_7914()) {
				temp.method_7939(storedItem.method_7914());
			}
			tagCompound.method_10566("storedStack", temp.method_7953(new class_2487()));
			tagCompound.method_10569("storedQuantity", Math.min(storedItem.method_7947(), maxCapacity));
		} else {
			tagCompound.method_10569("storedQuantity", 0);
		}
		inventory.write(tagCompound);
		return tagCompound;
	}

	public class_1799 getDropWithNBT() {
		class_2487 blockEntity = new class_2487();
		class_1799 dropStack = new class_1799(getBlockType(), 1);
		writeWithoutCoords(blockEntity);
		dropStack.method_7980(new class_2487());
		dropStack.method_7969().method_10566("blockEntity", blockEntity);
		storedItem.method_7939(0);
		inventory.method_5447(1, class_1799.field_8037);
		syncWithAll();

		return dropStack;
	}

	public int getStoredCount() {
		return storedItem.method_7947();
	}

	public List<class_1799> getContentDrops() {
		ArrayList<class_1799> stacks = new ArrayList<>();

		if (!getStoredItemType().method_7960()) {
			if (!inventory.method_5438(1).method_7960()) {
				stacks.add(inventory.method_5438(1));
			}
			int size = storedItem.method_7914();
			for (int i = 0; i < getStoredCount() / size; i++) {
				class_1799 droped = storedItem.method_7972();
				droped.method_7939(size);
				stacks.add(droped);
			}
			if (getStoredCount() % size != 0) {
				class_1799 droped = storedItem.method_7972();
				droped.method_7939(getStoredCount() % size);
				stacks.add(droped);
			}
		}

		return stacks;
	}

	// TileMachineBase
	@Override
	public void method_16896() {
		super.method_16896();
		if (!field_11863.field_9236) {
			class_1799 outputStack = class_1799.field_8037;
			if (!inventory.method_5438(1).method_7960()) {
				outputStack = inventory.method_5438(1);
			}
			if (!inventory.method_5438(0).method_7960()
					&& (storedItem.method_7947() + outputStack.method_7947()) < maxCapacity) {
				class_1799 inputStack = inventory.method_5438(0);
				if (getStoredItemType().method_7960()
						|| (storedItem.method_7960() && ItemUtils.isItemEqual(inputStack, outputStack, true, true))) {

					storedItem = inputStack;
					inventory.method_5447(0, class_1799.field_8037);
				} else if (ItemUtils.isItemEqual(getStoredItemType(), inputStack, true, true)) {
					int reminder = maxCapacity - storedItem.method_7947() - outputStack.method_7947();
					if (inputStack.method_7947() <= reminder) {
						setStoredItemCount(inputStack.method_7947());
						inventory.method_5447(0, class_1799.field_8037);
					} else {
						setStoredItemCount(maxCapacity - outputStack.method_7947());
						inventory.method_5438(0).method_7934(reminder);
					}
				}
				method_5431();
				syncWithAll();
			}

			if (!storedItem.method_7960()) {
				if (outputStack.method_7960()) {

					class_1799 delivered = storedItem.method_7972();
					delivered.method_7939(Math.min(storedItem.method_7947(), delivered.method_7914()));
					storedItem.method_7934(delivered.method_7947());

					if (storedItem.method_7960()) {
						storedItem = class_1799.field_8037;
					}

					inventory.method_5447(1, delivered);
					method_5431();
					syncWithAll();
				} else if (ItemUtils.isItemEqual(storedItem, outputStack, true, true)
						&& outputStack.method_7947() < outputStack.method_7914()) {

					int wanted = Math.min(storedItem.method_7947(),
							outputStack.method_7914() - outputStack.method_7947());
					outputStack.method_7939(outputStack.method_7947() + wanted);
					storedItem.method_7934(wanted);

					if (storedItem.method_7960()) {
						storedItem = class_1799.field_8037;
					}
					method_5431();
					syncWithAll();
				}
			}
		}
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	@Override
	public void method_11014(class_2487 tagCompound) {
		super.method_11014(tagCompound);
		readWithoutCoords(tagCompound);
	}

	@Override
	public class_2487 method_11007(class_2487 tagCompound) {
		super.method_11007(tagCompound);
		writeWithoutCoords(tagCompound);
		return tagCompound;
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<TechStorageBaseBlockEntity> getInventory() {
		return inventory;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 entityPlayer) {
		return getDropWithNBT();
	}

	// IListInfoProvider
	@Override
	public void addInfo(List<class_2561> info, boolean isReal, boolean hasData) {
		if (isReal || hasData) {
			int size = 0;
			String name = "of nothing";
			if (!storedItem.method_7960()) {
				name = storedItem.method_7964().getString();
				size += storedItem.method_7947();
			}
			if (!inventory.method_5438(1).method_7960()) {
				name = inventory.method_5438(1).method_7964().getString();
				size += inventory.method_5438(1).method_7947();
			}
			info.add(new class_2585(size + " " + name));
		}
	}

	public class_1799 getStoredItemType() {
		return storedItem.method_7960() ? inventory.method_5438(1) : storedItem;
	}


	public void setStoredItemCount(int amount) {
		storedItem.method_7933(amount);
		method_5431();
	}

	public void setStoredItemType(class_1799 type, int amount) {
		storedItem = type;
		storedItem.method_7939(amount);
		method_5431();
	}

	public int getMaxStoredCount() {
		return maxCapacity;
	}
}
