/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.iron;

import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.util.RebornInventory;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;
import techreborn.utils.RecipeUtils;

import java.util.Optional;
import net.minecraft.class_1303;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_3222;
import net.minecraft.class_3861;
import net.minecraft.class_3956;

public class IronFurnaceBlockEntity extends AbstractIronMachineBlockEntity implements IContainerProvider {

	int inputSlot = 0;
	int outputSlot = 1;
	public float experience;
	
	public IronFurnaceBlockEntity() {
		super(TRBlockEntities.IRON_FURNACE, 2, TRContent.Machine.IRON_FURNACE.block);
		this.inventory = new RebornInventory<>(3, "IronFurnaceBlockEntity", 64, this);
	}
	
	public void handleGuiInputFromClient(class_1657 playerIn) {
		if (playerIn instanceof class_3222) {
			class_3222 player = (class_3222) playerIn;
			int totalExperience = (int) experience;
			while (totalExperience > 0) {
				int expToDrop = class_1303.method_5918(totalExperience);
				totalExperience -= expToDrop;
				player.field_6002.method_8649(new class_1303(player.field_6002, player.method_23317(), player.method_23318() + 0.5D, player.method_23321() + 0.5D, expToDrop));
			}
		}
		experience = 0;
	}
	
	private class_1799 getResultFor(class_1799 stack) {
		class_1799 result = RecipeUtils.getMatchingRecipes(field_11863, class_3956.field_17546, stack);
		if (!result.method_7960()) {
			return result.method_7972();
		}
		return class_1799.field_8037;
	}

	private float getExperienceFor(class_1799 stack) {
		Optional<class_3861> recipe = field_11863.method_8433().method_8132(class_3956.field_17546, this, field_11863);
		if (recipe.isPresent()) {
			return recipe.get().method_8171();
		}
		return 0;
	}
	
	// AbstractIronMachineBlockEntity
	@Override
	protected void smelt() {
		if (!canSmelt()) {
			return;
		}
		class_1799 inputStack = inventory.method_5438(inputSlot);
		class_1799 resultStack = getResultFor(inputStack);
		
		if (inventory.method_5438(outputSlot).method_7960()) {
			inventory.method_5447(outputSlot, resultStack.method_7972());
		} else if (inventory.method_5438(outputSlot).method_7962(resultStack)) {
			inventory.method_5438(outputSlot).method_7933(resultStack.method_7947());
		}
		experience += getExperienceFor(inputStack);
		if (inputStack.method_7947() > 1) {
			inventory.shrinkSlot(inputSlot, 1);
		} else {
			inventory.method_5447(inputSlot, class_1799.field_8037);
		}
	}
	
	@Override
	protected boolean canSmelt() {
		if (inventory.method_5438(inputSlot).method_7960()) {
			return false;
		}		
		class_1799 outputStack = getResultFor(inventory.method_5438(inputSlot));
		if (outputStack.method_7960())
			return false;
		if (inventory.method_5438(outputSlot).method_7960())
			return true;
		if (!inventory.method_5438(outputSlot).method_7962(outputStack))
			return false;
		int result = inventory.method_5438(outputSlot).method_7947() + outputStack.method_7947();
		return result <= inventory.getStackLimit() && result <= outputStack.method_7914();
	}

	@Override
	public boolean isStackValid(int slotID, class_1799 stack) {
		return !getResultFor(stack).method_7960();
	}

	@Override
	public void method_11014(class_2487 compoundTag) {
		super.method_11014(compoundTag);
		experience = compoundTag.method_10583("Experience");
	}
	
	@Override
	public class_2487 method_11007(class_2487 compoundTag) {
		 super.method_11007(compoundTag);
		 compoundTag.method_10548("Experience", experience);
		 return compoundTag;
	}

	// IContainerProvider
	public float getExperience() {
		return experience;
	}
	
	public void setExperience(float experience) {
		this.experience = experience;
	}

	@Override
	public int[] getInputSlots() {
		return new int[]{inputSlot};
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("ironfurnace").player(player.field_7514).inventory().hotbar()
				.addInventory().blockEntity(this)
				.fuelSlot(2, 56, 53).slot(0, 56, 17).outputSlot(1, 116, 35)
				.sync(this::getBurnTime, this::setBurnTime)
				.sync(this::getProgress, this::setProgress)
				.sync(this::getTotalBurnTime, this::setTotalBurnTime)
				.sync(this::getExperience, this::setExperience)
				.addInventory().create(this, syncID);
	}
}
