/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.gui;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_310;
import net.minecraft.class_4493;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.gui.builder.GuiBase;
import reborncore.client.gui.builder.widget.GuiButtonSimple;
import reborncore.client.gui.guibuilder.GuiBuilder;
import reborncore.common.network.NetworkManager;
import techreborn.blockentity.machine.iron.IronFurnaceBlockEntity;
import techreborn.packets.ServerboundPackets;
import techreborn.utils.PlayerUtils;

public class GuiIronFurnace extends GuiBase<BuiltContainer> {

	IronFurnaceBlockEntity blockEntity;

	public GuiIronFurnace(int syncID, class_1657 player, IronFurnaceBlockEntity furnace) {
		super(player, furnace,  furnace.createContainer(syncID, player));
		this.blockEntity = furnace;
	}
	
	public void onClick(){
		NetworkManager.sendToServer(ServerboundPackets.createPacketExperience(blockEntity));
	}	
	
	@Override
	public void init() {
		super.init();
		addButton(new GuiButtonSimple(getGuiLeft() + 116, getGuiTop() + 57, 18, 18, "", b -> onClick()) {

			@Override
			public void renderToolTip(int mouseX, int mouseY) {
				class_1657 player = field_17410.field_7546;
				if (player == null) {
					return;
				}
				String message = "Experience: ";

				float furnaceExp = blockEntity.experience;
				if (furnaceExp <= 0) {
					message = message + "0";
				} else {
					float expTillLevel = (1.0F - player.field_7510) * player.method_7349();
					if (furnaceExp <= expTillLevel) {
						int percentage = (int) (blockEntity.experience * 100 / player.method_7349());
						message = message + "+"
								+ (percentage > 0 ? String.valueOf(percentage) : "<1")
								+ "%";
					} else {
						int levels = 0;
						furnaceExp -= expTillLevel;
						while (furnaceExp > 0) {
							furnaceExp -= PlayerUtils.getLevelExperience(player.field_7520);
							++levels;
						}
						message = message + "+" + String.valueOf(levels) + "L";
					}
				}

				List<String> list = new ArrayList<>();				
				list.add(message);
				renderTooltip(list, mouseX, mouseY);
				class_4493.method_22034();
				class_4493.method_22000(1, 1, 1, 1);				
					
			}
			
			@Override
			public void renderBg(class_310 mc, int mouseX, int mouseY) {
				mc.method_1480().method_4023(new class_1799(class_1802.field_8287), x, y);
			}
		});		
	}
	
	@Override
	protected void method_2389(float lastFrameDuration, int mouseX, int mouseY) {
		super.method_2389(lastFrameDuration, mouseX, mouseY);
		final GuiBase.Layer layer = GuiBase.Layer.BACKGROUND;

		// Input slot
		drawSlot(56, 17, layer);
		// Fuel slot
		drawSlot(56, 53, layer);

		drawOutputSlot(116, 35, layer);
	}
	
	@Override
	protected void method_2388(int mouseX, int mouseY) {
		super.method_2388(mouseX, mouseY);
		final GuiBase.Layer layer = GuiBase.Layer.FOREGROUND;

		builder.drawProgressBar(this, blockEntity.getProgressScaled(100), 100, 85, 36, mouseX, mouseY, GuiBuilder.ProgressDirection.RIGHT, layer);
		builder.drawBurnBar(this, blockEntity.getBurnTimeRemainingScaled(100), 100, 56, 36, mouseX, mouseY, layer);
	}
}
